from pydantic import BaseModel
import inspect
import ast
import os

class Config(BaseModel):

    # NOTE: should be in sync with ui.run arguments
    host: str = '0.0.0.0'
    port: int = 80
    title: str = 'NiceGUI'
    favicon: str = 'favicon.ico'
    reload: bool = True
    show: bool = True


excluded_endings = (
    '<string>',
    'spawn.py',
    'runpy.py',
    os.path.join('debugpy', 'server', 'cli.py'),
    os.path.join('debugpy', '__main__.py'),
)
for f in reversed(inspect.stack()):
    if not any(f.filename.endswith(ending) for ending in excluded_endings):
        filepath = f.filename
        break
else:
    raise Exception("Could not find main script in stacktrace")

with open(filepath) as f:
    source = f.read()

for node in ast.parse(source).body:
    try:
        func = node.value.func
        if func.value.id == 'ui' and func.attr == 'run':
            args = {
                keyword.arg:
                    keyword.value.n if isinstance(keyword.value, ast.Num) else
                    keyword.value.s if isinstance(keyword.value, ast.Str) else
                    keyword.value.value
                for keyword in node.value.keywords
            }
            config = Config(**args)
            break
    except AttributeError:
        continue
else:
    raise Exception('Could not find ui.run() command')

os.environ['HOST'] = config.host
os.environ['PORT'] = str(config.port)
os.environ["STATIC_DIRECTORY"] = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'static')
os.environ["TEMPLATES_DIRECTORY"] = os.path.join(os.environ["STATIC_DIRECTORY"], 'templates')



import justpy as jp
from .element import Element

class Svg(Element):

    def __init__(self,
                 content: str = '',
                 ):
        """Svg Element

        Displays an svg.

        :param content: the svg definition
        """

        view = jp.Div(style="padding:0;width:100%;height:100%")
        super().__init__(view)
        self.content = content

    @property
    def content(self):

        return self.view.inner_html()

    @content.setter
    def content(self, content: any):

        self.view.components = []
        jp.parse_html(content, a=self.view)

    def set_content(self, content: str):

        self.content = content

    def bind_content_to(self, target, forward=lambda x: x):

        self.content.bind_to(target, forward=forward, nesting=1)
        return self

    def bind_content_from(self, target, backward=lambda x: x):

        self.content.bind_from(target, backward=backward, nesting=1)
        return self

    def bind_content(self, target, forward=lambda x: x, backward=lambda x: x):

        self.content.bind(target, forward=forward, backward=backward, nesting=1)
        return self



from .element import Element

class Group(Element):

    def __enter__(self):

        self.view_stack.append(self.view)
        return self

    def __exit__(self, *_):

        self.view_stack.pop()



import justpy as jp
import matplotlib.pyplot as plt
from .element import Element

class Plot(Element):

    def __init__(self,
                 *,
                 close: bool = True,
                 **kwargs,
                 ):
        """Plot Context

        Create a context to configure a `Matplotlib <https://matplotlib.org/>`_ plot.

        :param close: whether the figure should be closed after exiting the context; set to `False` if you want to update it later, default is `True`
        :param kwargs: arguments like `figsize` which should be passed to `pyplot.figure <https://matplotlib.org/stable/api/_as_gen/matplotlib.pyplot.figure.html>`_
        """

        self.close = close
        self.fig = plt.figure(**kwargs)

        view = jp.Matplotlib()
        view.set_figure(self.fig)

        super().__init__(view)

    def __enter__(self):

        plt.figure(self.fig)

        return self

    def __exit__(self, *_):

        self.view.set_figure(plt.gcf())

        if self.close:
            plt.close(self.fig)



import justpy as jp
from typing import Callable, List, Dict, Union
from .choice_element import ChoiceElement

class Select(ChoiceElement):

    def __init__(self,
                 options: Union[List, Dict],
                 *,
                 value: any = None,
                 on_change: Callable = None,
                 ):
        """Dropdown Selection Element

        :param options: a list ['value1', ...] or dictionary `{'value1':'label1', ...}` specifying the options
        :param value: the inital value
        :param on_change: callback to execute when selection changes
        """

        view = jp.QSelect(options=options, input=self.handle_change)

        super().__init__(view, options, value=value, on_change=on_change)

    def value_to_view(self, value: any):

        matches = [o for o in self.view.options if o['value'] == value]
        if any(matches):
            return matches[0]['label']
        else:
            return value

    def handle_change(self, msg):

        msg['label'] = msg['value']['label']
        msg['value'] = msg['value']['value']
        return super().handle_change(msg)



import justpy as jp
from .group import Group

class Column(Group):

    def __init__(self):

        view = jp.QDiv(classes='column items-start', style='gap: 1em', delete_flag=False)

        super().__init__(view)



from typing import Callable
import justpy as jp
from .element import Element
from ..utils import handle_exceptions, provide_arguments

class Button(Element):

    def __init__(self,
                 text: str = '',
                 *,
                 on_click: Callable = None,
                 ):
        """Button Element

        :param text: the label of the button
        :param on_click: callback which is invoked when button is pressed
        """

        view = jp.QButton(label=text, color='primary')

        if on_click is not None:
            view.on('click', handle_exceptions(provide_arguments(on_click)))

        super().__init__(view)



import justpy as jp
from binding.binding import BindableProperty

class Element:

    wp: None
    view_stack = []

    visible = BindableProperty

    def __init__(self,
                 view: jp.HTMLBaseComponent,
                 ):

        self.parent_view = self.view_stack[-1]
        self.parent_view.add(view)
        view.add_page(self.wp)
        self.view = view

        self.visible = True

    @property
    def visible(self):

        return self.visible_

    @visible.setter
    def visible(self, visible: bool):

        self.visible_ = visible
        (self.view.remove_class if self.visible_ else self.view.set_class)('hidden')

    def bind_visibility_to(self, target, forward=lambda x: x):

        self.visible.bind_to(target, forward=forward, nesting=1)
        return self

    def bind_visibility_from(self, target, backward=lambda x: x, *, value=None):

        if value is not None:
            def backward(x): return x == value

        self.visible.bind_from(target, backward=backward, nesting=1)
        return self

    def bind_visibility(self, target, forward=lambda x: x, backward=None, *, value=None):

        if value is not None:
            def backward(x): return x == value

        self.visible.bind(target, forward=forward, backward=backward, nesting=1)
        return self

    def classes(self, add: str = '', *, remove: str = '', replace: str = ''):

        class_list = [] if replace else self.view.classes.split()
        class_list = [c for c in class_list if c not in remove]
        class_list += add.split()
        class_list += replace.split()
        self.view.classes = ' '.join(class_list)

        return self

    def style(self, add: str = '', *, remove: str = '', replace: str = ''):

        style_list = [] if replace else self.view.style.split(';')
        style_list = [c for c in style_list if c not in remove.split(';')]
        style_list += add.split(';')
        style_list += replace.split(';')
        self.view.style = ';'.join(style_list)

        return self

    def props(self, add: str = '', *, remove: str = '', replace: str = ''):

        for prop in remove.split() + replace.split():
            setattr(self.view, prop.split('=')[0], None)

        for prop in add.split() + replace.split():
            if '=' in prop:
                setattr(self.view, *prop.split('='))
            else:
                setattr(self.view, prop, True)

        return self



import markdown2
from .html import Html
import re

class Markdown(Html):

    def __init__(self,
                 content: str = '',
                 ):
        """Markdown Element

        Renders markdown onto the page.

        :param content: the markdown content to be displayed
        """

        super().__init__(content)

    def set_content(self, content: str):

        html = markdown2.markdown(content, extras=['fenced-code-blocks'])
        # we need explicit markdown styling because tailwind css removes all default styles
        html = Markdown.apply_tailwind(html)
        super().set_content(html)

    @staticmethod
    def apply_tailwind(html: str):

        rep = {
            '<h1': '<h1 class="text-5xl mb-4 mt-6"',
            '<h2': '<h2 class="text-4xl mb-3 mt-5"',
            '<h3': '<h3 class="text-3xl mb-2 mt-4"',
            '<h4': '<h4 class="text-2xl mb-1 mt-3"',
            '<h5': '<h5 class="text-1xl mb-0.5 mt-2"',
            '<a': '<a class="underline text-blue-600 hover:text-blue-800 visited:text-purple-600"',
            '<ul': '<ul class="list-disc ml-6"',
            '<p>': '<p class="mb-2">',
            '<div\ class="codehilite">': '<div class=" codehilite mb-2 p-2" style="overflow: scroll">',
            '<code': '<code style="background-color: #f8f8f8"',
        }
        pattern = re.compile("|".join(rep.keys()))
        return pattern.sub(lambda m: rep[re.escape(m.group(0))], html)



import justpy as jp
import os.path
from starlette.routing import Route
from starlette.responses import FileResponse

class CustomView(jp.JustpyBaseComponent):

    vue_dependencies = []

    def __init__(self, vue_type, filepath, dependencies=[], **options):

        self.vue_type = vue_type
        self.vue_filepath = os.path.realpath(filepath).replace('.py', '.js')
        self.vue_dependencies = dependencies

        self.pages = {}
        self.classes = ''
        self.options = jp.Dict(**options)

        super().__init__()

        self.initialize(temp=False)

    def add_page(self, wp: jp.WebPage):

        marker = '<!--' + self.__module__ + '-->\n'
        if marker not in wp.head_html:
            wp.head_html += marker
            for dependency in self.vue_dependencies:
                if dependency.startswith('http://') or dependency.startswith('https://'):
                    wp.head_html += f'<script src="{dependency}"></script>\n'
                else:
                    wp.head_html += f'<script src="lib/{dependency}"></script>\n'
                    filepath = f'{os.path.dirname(self.vue_filepath)}/lib/{dependency}'
                    route = Route(f'/lib/{dependency}', lambda _, filepath=filepath: FileResponse(filepath))
                    jp.app.routes.insert(0, route)

        if self.vue_filepath not in jp.component_file_list:
            filename = os.path.basename(self.vue_filepath)
            jp.app.routes.insert(0, Route(f'/{filename}', lambda _: FileResponse(self.vue_filepath)))
            jp.component_file_list += [filename]

        super().add_page(wp)

    def react(self, _):

        pass

    def convert_object_to_dict(self):

        return {
            'vue_type': self.vue_type,
            'id': self.id,
            'show': True,
            'options': self.options,
        }



import justpy as jp
from .group import Group

class Image(Group):

    def __init__(self,
                 source: str = '',
                 ):
        """Image Element

        Displays an image.

        :param source: the source of the image; can be an url or a base64 string
        """

        view = jp.QImg(src=source)

        super().__init__(view)

    @property
    def source(self):

        return self.view.src

    @source.setter
    def source(self, source: any):

        self.view.src = source

    def set_source(self, source: str):

        self.source = source

    def bind_source_to(self, target, forward=lambda x: x):

        self.source.bind_to(target, forward=forward, nesting=1)
        return self

    def bind_source_from(self, target, backward=lambda x: x):

        self.source.bind_from(target, backward=backward, nesting=1)
        return self

    def bind_source(self, target, forward=lambda x: x, backward=lambda x: x):

        self.source.bind(target, forward=forward, backward=backward, nesting=1)
        return self



import justpy as jp
from typing import Callable
import base64
from .element import Element
from ..utils import handle_exceptions

class Upload(Element):

    def __init__(self,
                 *,
                 multiple: bool = False,
                 on_upload: Callable = None,
                 ):
        """File Upload Element

        :param multiple: allow uploading multiple files at once (default: False)
        :param on_upload: callback to execute when a file is uploaded (list of bytearrays)
        """
        self.on_upload = handle_exceptions(on_upload)
        view = jp.Form(enctype='multipart/form-data', classes='flex gap-4 items-center',
                       submit=lambda s, m: self.submit(s, m))
        jp.Input(type='file', multiple=multiple, a=view)
        jp.QButton(type='submit', text='Upload', color='primary', a=view)

        super().__init__(view)

    def submit(self, _, msg):

        for form_data in msg.form_data:
            if form_data.type == 'file':
                self.on_upload([base64.b64decode(f.file_content) for f in form_data.files])



from typing import Callable
import justpy as jp
from .float_element import FloatElement

class Slider(FloatElement):

    def __init__(self,
                 *,
                 min: float,
                 max: float,
                 step: float = 1,
                 value: float = None,
                 on_change: Callable = None,
                 ):
        """Slider Element

        :param min: lower bound of the slider
        :param max: upper bound of the slider
        :param step: step size
        :param value: inital value to set position of the slider
        :param on_change: callback which is invoked when the user releases the slider
        """
        view = jp.QSlider(min=min, max=max, step=step, change=self.handle_change)

        super().__init__(view, value=value, on_change=on_change)



import justpy as jp
from .element import Element

class Link(Element):

    def __init__(self,
                 text: str = '',
                 href: str = '#',
                 ):

        view = jp.A(text=text, href=href)

        super().__init__(view)



import justpy as jp
from typing import Callable
from .value_element import ValueElement

class FloatElement(ValueElement):

    def __init__(self,
                 view: jp.HTMLBaseComponent,
                 *,
                 value: float,
                 format: str = None,
                 on_change: Callable,
                 ):

        self.format = format

        super().__init__(view, value=value, on_change=on_change)

    def value_to_view(self, value: float):

        if value is None:
            return None
        elif self.format is None:
            return str(value)
        else:
            return self.format % float(value)



import justpy as jp
from .element import Element

class Label(Element):

    def __init__(self,
                 text: str = '',
                 ):
        """Label Element

        Displays some text.

        :param text: the content of the label
        """

        view = jp.Div(text=text)

        super().__init__(view)

    @property
    def text(self):

        return self.view.text

    @text.setter
    def text(self, text: any):

        self.view.text = text

    def set_text(self, text: str):

        self.text = text

    def bind_text_to(self, target, forward=lambda x: x):

        self.text.bind_to(target, forward=forward, nesting=1)
        return self

    def bind_text_from(self, target, backward=lambda x: x):

        self.text.bind_from(target, backward=backward, nesting=1)
        return self

    def bind_text(self, target, forward=lambda x: x, backward=lambda x: x):

        self.text.bind(target, forward=forward, backward=backward, nesting=1)
        return self



import justpy as jp
from typing import Any, Callable
import traceback
from binding import BindableProperty
from .element import Element
from ..utils import EventArguments

class ValueElement(Element):

    value = BindableProperty()

    def __init__(self,
                 view: jp.HTMLBaseComponent,
                 *,
                 value: Any,
                 on_change: Callable,
                 ):

        super().__init__(view)

        self.on_change = on_change
        self.value = value
        self.value.bind_to(self.view.value, forward=self.value_to_view)

    def value_to_view(self, value):

        return value

    def handle_change(self, msg):

        self.value = msg['value']

        if self.on_change is not None:
            try:
                try:
                    self.on_change()
                except TypeError:
                    self.on_change(EventArguments(self, **msg))
            except Exception:
                traceback.print_exc()

    def bind_value_to(self, target, forward=lambda x: x):

        self.value.bind_to(target, forward=forward, nesting=1)
        return self

    def bind_value_from(self, target, backward=lambda x: x):

        self.value.bind_from(target, backward=backward, nesting=1)
        return self

    def bind_value(self, target, forward=lambda x: x, backward=lambda x: x):

        self.value.bind(target, forward=forward, backward=backward, nesting=1)
        return self



import traceback

class EventArguments:

    def __init__(self, sender, **kwargs):

        self.sender = sender
        for key, value in kwargs.items():
            setattr(self, key, value)

def provide_arguments(func, *keys):
    def inner_function(sender, event):
        try:
            return func()
        except TypeError:
            return func(EventArguments(sender, **{key: event[key] for key in keys}))
    return inner_function

def handle_exceptions(func):
    def inner_function(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except Exception:
            traceback.print_exc()
    return inner_function



#!/usr/bin/env python3
from typing import Awaitable, Callable
import asyncio
import binding
from pygments.formatters import HtmlFormatter
from .ui import Ui  # NOTE: before justpy
import justpy as jp
from .elements.element import Element
from .timer import Timer

wp = jp.QuasarPage(delete_flag=False, title=Ui.config.title, favicon=Ui.config.favicon)
wp.tailwind = True  # use Tailwind classes instead of Quasars
wp.css = HtmlFormatter().get_style_defs('.codehilite')
wp.head_html += '<script>confirm = () => true;</script>\n'  # avoid confirmation dialog for reload

main = jp.Div(a=wp, classes='q-ma-md column items-start', style='row-gap: 1em')
main.add_page(wp)

jp.justpy(lambda: wp, start_server=False)

async def binding_loop():
    while True:
        binding.update()
        await asyncio.sleep(0.1)

def create_task(coro):
    loop = asyncio.get_event_loop()
    return loop.create_task(coro)

tasks = []

@jp.app.on_event('startup')
def startup():
    tasks.extend(create_task(t) for t in Timer.tasks)
    tasks.extend(create_task(t) for t in Ui.startup_tasks if isinstance(t, Awaitable))
    [t() for t in Ui.startup_tasks if isinstance(t, Callable)]
    jp.run_task(binding_loop())

@jp.app.on_event('shutdown')
def shutdown():
    [create_task(t) for t in Ui.shutdown_tasks if isinstance(t, Awaitable)]
    [t() for t in Ui.shutdown_tasks if isinstance(t, Callable)]
    [t.cancel() for t in tasks]

Element.wp = wp
Element.view_stack = [main]

app = jp.app
ui = Ui()



import inspect
import sys
import webbrowser
import uvicorn
from .config import config  # NOTE: before justpy
import justpy as jp

if config.reload and not inspect.stack()[-2].filename.endswith('spawn.py'):

    if config.show:
        webbrowser.open(f'http://{config.host}:{config.port}/')
    uvicorn.run('nicegui:app', host=config.host, port=config.port, lifespan='on', reload=True)
    sys.exit()

def run(self, *, host='0.0.0.0', port=80, title='NiceGUI', favicon='favicon.ico', reload=True, show=True):

    if reload == False:  # NOTE: in case reload == True we already started uvicorn above
        if show:
            webbrowser.open(f'http://{host if host != "0.0.0.0" else "127.0.0.1"}:{port}/')
        uvicorn.run(jp.app, host=host, port=port, lifespan='on')



import asyncio
import time
import traceback
from typing import Awaitable
from binding import BindableProperty
from .elements.element import Element
from .utils import handle_exceptions

class Timer:

    tasks = []

    active = BindableProperty

    def __init__(self, interval: float, callback: Awaitable, *, active: bool = True, once: bool = False):
        """Timer

        One major drive behind the creation of NiceGUI was the necessity to have a simple approach to update the interface in regular intervals, for example to show a graph with incomming measurements.
        A timer will execute a callback repeatedly with a given interval.
        The parent view container will be updated automatically, as long as the callback does not return `False`.

        :param interval: the interval in which the timer is called
        :param callback: function to execute when interval elapses (can return `False` to prevent view update)
        :param active: whether the callback should be executed or not
        :param once: whether the callback is only executed once after a delay specified by `interval`; default is `False`
        """

        parent = Element.view_stack[-1]
        self.active = active

        async def timeout():

            await asyncio.sleep(interval)
            handle_exceptions(callback)()
            await parent.update()

        async def loop():

            while True:
                try:
                    start = time.time()
                    if self.active:
                        needs_update = handle_exceptions(callback)()
                        if needs_update != False:
                            await parent.update()
                    dt = time.time() - start
                    await asyncio.sleep(interval - dt)
                except asyncio.CancelledError:
                    pass
                except:
                    traceback.print_exc()
                    await asyncio.sleep(interval)

        self.tasks.append(timeout() if once else loop())



from typing import Callable
import justpy as jp
from .bool_element import BoolElement

class Checkbox(BoolElement):

    def __init__(self,
                 text: str = '',
                 *,
                 value: bool = False,
                 on_change: Callable = None,
                 ):
        """Checkbox Element

        :param text: the label to display next to the checkbox
        :param value: set to `True` if it should be checked initally; default is `False`
        :param on_change: callback to execute when value changes
        """
        view = jp.QCheckbox(text=text, input=self.handle_change)

        super().__init__(view, value=value, on_change=on_change)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['nicegui', 'nicegui.elements']

package_data = \
{'': ['*'],
 'nicegui': ['static/*',
             'static/templates/*',
             'static/templates/css/*',
             'static/templates/highcharts/*',
             'static/templates/js/*',
             'static/templates/local/*',
             'static/templates/local/fontawesome/css/*',
             'static/templates/local/fontawesome/webfonts/*',
             'static/templates/local/ionicons/*',
             'static/templates/local/materialdesignicons/iconfont/*',
             'static/templates/local/robotofont/*',
             'static/templates/local/tailwind/*'],
 'nicegui.elements': ['lib/*']}

install_requires = \
['Pygments>=2.9.0,<3.0.0',
 'asttokens>=2.0.5,<3.0.0',
 'binding>=0.3.1,<0.4.0',
 'docutils>=0.17.1,<0.18.0',
 'justpy==0.1.5',
 'markdown2>=2.4.0,<3.0.0',
 'matplotlib>=3.4.1,<4.0.0',
 'pydantic>=1.8.2,<2.0.0',
 'typing-extensions>=3.10.0,<4.0.0',
 'uvicorn[watchgodreloader]>=0.14.0,<0.15.0',
 'watchgod>=0.7,<0.8',
 'websockets>=9.1,<10.0']

setup_kwargs = {
    'name': 'nicegui',
    'version': '0.4.13',
    'description': 'High-Level Abstraction Web-GUI Using Just Python',
    'long_description': '# NiceGUI\n\n<img src="https://raw.githubusercontent.com/zauberzeug/nicegui/main/sceenshots/ui-elements.png" width="300" align="right">\n\nWe like [Streamlit](https://streamlit.io/) but find it does [too much magic when it comes to state handling](https://github.com/zauberzeug/nicegui/issues/1#issuecomment-847413651).\nIn search for an alernative nice library to write simple graphical user interfaces in Python we discovered [justpy](https://justpy.io/).\nWhile too "low-level HTML" for our daily usage it provides a great basis for "NiceGUI".\n\n## Purpose\n\nNiceGUI is intended to be used for small scripts and single-page user interfaces with a very limited user base.\nLike smart home solutions, micro web apps or robotics projects.\nIt\'s also helpful for development, when tweaking/configuring a machine learning training or tuning motor controllers.\n\n## Features\n\n- browser-based graphical user interface\n- shared state between multiple browser windows\n- implicit reload on code change\n- clean set of GUI elements (label, button, checkbox, switch, slider, input, ...)\n- simple grouping with rows, columns and cards\n- general-purpose HTML and markdown elements\n- built-in timer to refresh data in intervals (even every 10 ms)\n- straight-forward data binding to write even less code\n\n## Installation\n\n```bash\npython3 -m pip install nicegui\n```\n\n## Usage\n\nWrite your nice GUI in a file `main.py`:\n\n```python\nfrom nicegui import ui\n\nui.label(\'Hello NiceGUI!\')\nui.button(\'BUTTON\', on_click=lambda: print(\'button was pressed\', flush=True))\n\nui.run()\n```\n\nLaunch it with:\n\n```bash\npython3 main.py\n```\n\nThe GUI is now avaliable through http://localhost/ in your browser.\nNote: The script will automatically reload the page when you modify the code.\n\n## Configuration\n\nYou can call `ui.run()` with optional arguments for some high-level configuration:\n\n- `host` (default: `\'0.0.0.0\'`)\n- `port` (default: `80`)\n- `title` (default: `\'NiceGUI\'`)\n- `favicon` (default: `\'favicon.ico\'`)\n- `reload`: automatically reload the ui on file changes (default: `True`)\n- `show`: automatically open the ui in a browser tab (default: `True`)\n\n## Docker\n\nUse the [multi-arch docker image](https://hub.docker.com/repository/docker/zauberzeug/nicegui) for pain-free installation:\n\n```bash\ndocker run --rm -p 8888:80 -v $(pwd)/my_script.py:/app/main.py -it zauberzeug/nicegui:latest\n```\n\nThis will start the server at http://localhost:8888 with code from `my_script.py` within the current directory.\nCode modification triggers an automatic reload.\n\n## API\n\nThe API reference is hosted at [https://nicegui.io](https://nicegui.io) and is [implemented with NiceGUI itself](https://github.com/zauberzeug/nicegui/blob/main/main.py).\nYou should also have a look at [examples.py](https://github.com/zauberzeug/nicegui/tree/main/examples.py) for an extensive demonstration of what you can do with NiceGUI.\n',
    'author': 'Zauberzeug GmbH',
    'author_email': 'info@zauberzeug.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/zauberzeug/nicegui',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



from typing import List
from .plot import Plot

class LinePlot(Plot):

    def __init__(self,
                 *,
                 n: int = 1,
                 limit: int = 100,
                 update_every: int = 1,
                 close: bool = True,
                 **kwargs,
                 ):
        """Line Plot

        Create a line plot. The `push` method provides live updating when utilized in combination with `ui.timer`.

        :param n: number of lines
        :param limit: maximum number of datapoints per line (new points will displace the oldest)
        :param update_every: update plot only after pushing new data multiple times to save CPU and bandwidth
        :param close: whether the figure should be closed after exiting the context; set to `False` if you want to update it later, default is `True`
        :param kwargs: arguments like `figsize` which should be passed to `pyplot.figure <https://matplotlib.org/stable/api/_as_gen/matplotlib.pyplot.figure.html>`_
        """

        super().__init__(close=close, **kwargs)

        self.x = []
        self.Y = [[] for _ in range(n)]
        self.lines = [self.fig.gca().plot([], [])[0] for _ in range(n)]
        self.slice = slice(0 if limit is None else -limit, None)
        self.update_every = update_every
        self.push_counter = 0

    def with_legend(self, titles: List[str], **kwargs):

        self.fig.gca().legend(titles, **kwargs)
        self.view.set_figure(self.fig)
        return self

    def push(self, x: List[float], Y: List[List[float]]):

        self.push_counter += 1

        self.x = [*self.x, *x][self.slice]
        for i in range(len(self.lines)):
            self.Y[i] = [*self.Y[i], *Y[i]][self.slice]

        if self.push_counter % self.update_every != 0:
            return

        for i in range(len(self.lines)):
            self.lines[i].set_xdata(self.x)
            self.lines[i].set_ydata(self.Y[i])

        flat_y = [y_i for y in self.Y for y_i in y]
        min_x = min(self.x)
        max_x = max(self.x)
        min_y = min(flat_y)
        max_y = max(flat_y)
        pad_x = 0.01 * (max_x - min_x)
        pad_y = 0.01 * (max_y - min_y)
        self.fig.gca().set_xlim(min_x - pad_x, max_x + pad_x)
        self.fig.gca().set_ylim(min_y - pad_y, max_y + pad_y)
        self.view.set_figure(self.fig)



import justpy as jp
from .group import Group

class Dialog(Group):

    def __init__(self,
                 *,
                 value: bool = False
                 ):
        """Dialog

        Creates a modal dialog.

        :param value: whether the dialog is already opened (default: False)
        """

        view = jp.QDialog(
            value=value,
            classes='row items-start bg-red-400',
            style='gap: 1em',
        )

        super().__init__(view)

    def open(self):

        self.view.value = True

    def close(self):

        self.view.value = False
